//
//  KextControl.m
//  PeerFilter
//
//  Created by U Kong Wong on 7/22/12.
//  Copyright (c) 2012 Eric U Kong Wong. All rights reserved.
//

#import "KextControl.h"

@implementation KextControl

@synthesize inputStream = _inputStream;

- (id)init
{
    _connected = false;
    _inputStream = nil;
    return [super init];
}

- (void)dealloc
{
    [self disconnect];
    [_inputStream release];
    [super dealloc];
}

- (BOOL)connect
{
    if (_connected) {
        [self disconnect];
    }
    
    // create system control socket
    _sock = socket(PF_SYSTEM, SOCK_DGRAM, SYSPROTO_CONTROL);
    if (_sock < 0) {
        return NO;
    }
    
    // get kext id and unit number using BUNDLE_ID
    struct ctl_info info;
    bzero(&info, sizeof(info));
    strlcpy(info.ctl_name, BUNDLE_ID, sizeof(info.ctl_name));
    
    if (ioctl(_sock, CTLIOCGINFO, &info) == -1) {
        return NO;      
    }
    
    // try to connect to kext, hoping that it has been loaded
    struct sockaddr_ctl sac;
    bzero(&sac, sizeof(sac));
    sac.sc_len = sizeof(sac);
    sac.sc_family = AF_SYSTEM;
    sac.ss_sysaddr = AF_SYS_CONTROL;
    sac.sc_id = info.ctl_id;
    sac.sc_unit = 0;
    
    int result = connect(_sock, (struct sockaddr *)&sac, sizeof(sac));
    
    if (result) {
        return NO;
    }
    
    // initialize NSInputStream
    CFReadStreamRef readStream;
    CFWriteStreamRef writeStream;
    CFStreamCreatePairWithSocket(NULL, _sock, &readStream, &writeStream);
    self.inputStream = (NSInputStream *)readStream;
    
    _connected = true;
    
    return YES;

}

- (void)disconnect
{
    [self stop];
    [self.inputStream close];
    close(_sock);
    _connected = false;   
}

- (int)setListTo: (struct pf_iprange_list) list
{
    int ret = -1;
    if (_connected) 
    {
        ret = setsockopt(_sock, SYSPROTO_CONTROL, PF_CONTROL_SET_LIST, &list, sizeof(list));
    }
    return ret;
}

- (int)start
{
    int ret = -1;
    if (_connected) 
    {
        ret = setsockopt(_sock, SYSPROTO_CONTROL, PF_CONTROL_START, NULL, 0);
    }
    return ret;
} 

- (int)stop
{
    int ret = -1;
    if (_connected) 
    {
        ret = setsockopt(_sock, SYSPROTO_CONTROL, PF_CONTROL_STOP, NULL, 0);
    }
    return ret;
} 

- (struct pf_stats)getStat 
{
    int ret = -1;
    struct pf_stats stats;
    if (_connected) 
    {
        unsigned int size = sizeof(stats);
        ret = getsockopt(_sock, SYSPROTO_CONTROL, PF_CONTROL_STAT, &stats, &size);
    }
    return stats;

}

- (BOOL)active
{
    int ret = -1;
    boolean_t active;
    if (_connected) 
    {
        unsigned int size = sizeof(active);
        ret = getsockopt(_sock, SYSPROTO_CONTROL, PF_CONTROL_ISACTIVE, &active, &size);
        if (ret != 0) {
            // lost connection
            [self disconnect];
            return NO;
        }
        else if (ret == 0) {
            return active;
        }
    }
    return NO;
}

- (BOOL)connected
{
    return _connected ? YES : NO;
}

@end
