/*
 *  KextControl.cpp
 *
 *  Copyright 2012 Eric U Kong Wong. All rights reserved.
 *
 *  This file is part of PeerFilter.
 *
 *  PeerFilter is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  PeerFilter is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with PeerFilter.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#include "KextControl.h"
#include <cstring>

KextControl::KextControl() {
    _sock = socket(PF_SYSTEM, SOCK_DGRAM, SYSPROTO_CONTROL);
    if (_sock < 0) {
        return;
    }
    
    struct ctl_info info;
    bzero(&info, sizeof(info));
    strlcpy(info.ctl_name, BUNDLE_ID, sizeof(info.ctl_name));
    
    if (ioctl(_sock, CTLIOCGINFO, &info) == -1)
        return;

    struct sockaddr_ctl sac;
    bzero(&sac, sizeof(sac));
    sac.sc_len = sizeof(sac);
    sac.sc_family = AF_SYSTEM;
    sac.ss_sysaddr = AF_SYS_CONTROL;
    sac.sc_id = info.ctl_id;
    sac.sc_unit = 0;
    
    int result = connect(_sock, (struct sockaddr *)&sac, sizeof(sac));
    
    if (result) {
        return;
    }
    _connected = true;
}

KextControl::~KextControl() {
    stop();
    close(_sock);
}

int KextControl::getStat(pf_stats &stats) {
    int ret = -1;
    if (_connected) {
        unsigned int size = sizeof(stats);
        ret = getsockopt(_sock, SYSPROTO_CONTROL, PF_CONTROL_STAT, &stats, &size);
    }
    return ret;
}

int KextControl::setRanges(pf_iprange_list const& r) {
    int ret = -1;
    if (_connected) {
        ret = setsockopt(_sock, SYSPROTO_CONTROL, PF_CONTROL_SET_LIST, &r, sizeof(r));
    }
    return ret;
}

void KextControl::resetRanges() {
    int ret = -1;
    if (_connected) {
        ret = setsockopt(_sock, SYSPROTO_CONTROL, PF_CONTROL_RESET, NULL, 0);
    }
} 

int KextControl::start() {
    int ret = -1;
    if (_connected) {
        ret = setsockopt(_sock, SYSPROTO_CONTROL, PF_CONTROL_START, NULL, 0);
    }
    return ret;
} 

int KextControl::stop() {
    int ret = -1;
    if (_connected) {
        ret = setsockopt(_sock, SYSPROTO_CONTROL, PF_CONTROL_STOP, NULL, 0);
    }
    return ret;
} 