/*
 *  ListReader.cpp
 *
 *  Copyright 2012 Eric U Kong Wong. All rights reserved.
 *
 *  This file is part of PeerFilter.
 *
 *  PeerFilter is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  PeerFilter is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with PeerFilter.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#include "ListReader.h"

#include <cstring>
#include <netinet/in.h>
#include <arpa/inet.h>

ListReader::ListReader(std::string filename) {
    file.open(filename.c_str(), std::ios::in);
    if(!file.is_open())
        throw std::runtime_error("unable to open file");
}

ListReader::~ListReader() {
    if (file.is_open())
        file.close();
}

inline uint32_t ip_to_uint(char const* ip) {
    struct in_addr sa;
    if (inet_pton(AF_INET, ip, &sa) > 0)
        return ntohl(sa.s_addr);
    else
        return 0;
}

pf_iprange* ListReader::next() {
    while (file.good()) {
        file.getline(buf, 2048); // is end of buf null terminated?
        char* ip_str = std::strrchr(buf, ':');
        if (ip_str) {
            *ip_str = '\0';
            ip_str++;
            char* ip_l = std::strtok(ip_str, "-");
            char* ip_h = std::strtok(NULL, "-");
            if (ip_l && ip_h) {
                current.start = ip_to_uint(ip_l);
                current.end = ip_to_uint(ip_h);
                if (current.start == 0 || current.end == 0)
                    continue;
                return &current;
            }
        }
    }
    return 0;
}
