/*
 *  main.cpp
 *
 *  Copyright 2012 Eric U Kong Wong. All rights reserved.
 *
 *  This file is part of PeerFilter.
 *
 *  PeerFilter is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  PeerFilter is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with PeerFilter.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#include <iostream>
#include <algorithm>
#include <vector>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "PeerFilterIpRange.h"
#include "ListReader.h"
#include "KextControl.h"

uint32_t ip_to_uint(char const* ip) {
    struct in_addr sa;
    if (inet_pton(AF_INET, ip, &sa) > 0)
        return ntohl(sa.s_addr);
    else
        return 0;
}

inline bool less_than(pf_iprange const& a, pf_iprange const& b) {
    return a.start < b.start;
}

inline bool adjacent(pf_iprange& a, pf_iprange& b) {
    if (a.start <= b.start && b.start <= a.end) {
        a.end = std::min(a.end, b.end);
        return true;
    }
    else {
        return false;
    }

}

int main() {
    ListReader reader("debug.p2p");
    pf_iprange* ipr;
    std::vector<pf_iprange> list;
    int c = 0;
    while ((ipr=reader.next())) {
        list.push_back(*ipr);
        std::cout << ipr->start << '-' << ipr->end << std::endl;
        c++;
    }
    std::cout << c << " add() called.\n" << list.size() << " IP ranges added\n";
    std::sort(list.begin(), list.end(), less_than);
    std::vector<pf_iprange>::iterator it;
    it = std::unique(list.begin(), list.end(), adjacent);
    list.resize(it - list.begin());
    pf_iprange_list p = { &list[0], list.size() };
    
    KextControl ctl;
    std::cout << "connected? " << ctl.connected() << std::endl;
    
    std::cout << "installing list " << ctl.setRanges(p) << std::endl;
    
    std::cout << "activating filter" << ctl.start() << std::endl;
    
    uint32_t ip;
    char input[16];
    while (std::cin >> input) {
        ip = ip_to_uint(input);
        pf_iprange const* cipr = pf_inranges(&p, ip);
        std::cout << (cipr!=NULL) << std::endl;
    }
    
    ctl.stop();
    return 0;
}
