//
//  main.c
//  PeerFilter
//
//  Created by U Kong Wong on 7/26/12.
//  Copyright 2012 Eric U Kong Wong. All rights reserved.
//
//
// This helper launches the KEXT. Launchd monitors a unix socket specified in the
// plist and launches this helper whenever there is an incoming connection.
// 

#include <sys/socket.h>
#include <stdio.h>
#include <launch.h>
#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <IOKit/kext/KextManager.h>
#include "PeerFilterKernControl.h"

static int sockfd = -1;

static void term_handler(int signum)
{
    printf("PeerFilterHelper signal received: %d\n", signum);
    if (sockfd != -1)
        close(sockfd);
    exit(0);
}

static int server_check_in(void)
{
    // check in
    launch_data_t req = launch_data_new_string(LAUNCH_KEY_CHECKIN);
    launch_data_t resp = launch_msg(req);
    launch_data_free(req);
    
    // get socket fd
    launch_data_t sockets = launch_data_dict_lookup(resp, LAUNCH_JOBKEY_SOCKETS);
    if (sockets == NULL)
        return -1;
    
    launch_data_t sarr = launch_data_dict_lookup(sockets, "com.github.mtrbean.PeerFilter.PeerFilterHelper");
    if (!sarr || launch_data_get_type(sarr) != LAUNCH_DATA_ARRAY)
        return -1;
            
    launch_data_t sock = launch_data_array_get_index(sarr, 0);
    if (!sock || launch_data_get_type(sock) != LAUNCH_DATA_FD)
        return -1;
    
    sockfd = launch_data_get_fd(sock);
    if (sockfd != -1) 
        fcntl(sockfd, F_SETFL, O_NONBLOCK);
    
	return sockfd;
}

static OSReturn load_kext(void)
{
    CFStringRef kextPath = CFSTR(KEXT_PATH);
    CFURLRef kextUrl = CFURLCreateWithFileSystemPath(kCFAllocatorDefault, kextPath, kCFURLPOSIXPathStyle, true);   
    OSReturn result = KextManagerLoadKextWithURL(kextUrl, NULL);
    CFRelease(kextUrl);
    CFRelease(kextPath);
    return result;
}

int main(int argc, const char * argv[])
{    
    signal(SIGTERM, term_handler);
    sockfd = server_check_in();
    printf("Helper checkin completed\n");
    
    int afd;
    if (sockfd != -1)
    {
        struct sockaddr saddr;
        socklen_t slen = 0;
        afd = accept(sockfd, (struct sockaddr *)&saddr, &slen);
        if (afd) {
            fcntl(afd, F_SETFL, O_NONBLOCK);
            
            OSReturn result = load_kext();
            if (result != kOSReturnSuccess) 
            {
                fprintf(stderr, "Cannot load kext from " KEXT_PATH "\n");
                return EXIT_FAILURE;
            }
            else {
                printf("Loaded kext from " KEXT_PATH "\n");
                char *msg = "loaded";
                write(afd, msg, strlen(msg));
            }  
            close(afd);
        }
        close(sockfd);
    }
    return EXIT_SUCCESS;
}