/*
 *  IpRange.c
 *
 *  Copyright 2012 Eric U Kong Wong. All rights reserved.
 *
 *  This file is part of PeerFilter.
 *
 *  PeerFilter is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  PeerFilter is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with PeerFilter.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#include "PeerFilterIpRange.h"

struct pf_iprange const* pf_inranges(struct pf_iprange_list const* ranges, 
                                     uint32_t ip) {
    unsigned int count = ranges->len;
    
    if (count <= 0) {
        return NULL;
    }
    
    unsigned int step;
    struct pf_iprange const* first = ranges->ranges;
    struct pf_iprange const* p = first;
    struct pf_iprange const* last = first + count;
    
    while (count>0) {
        p = first;
        step=count/2;
        p+=step;
        if (p->start < ip) {
            first = ++p;
            count -= step+1;
        }
        else count = step;
    }
    
    if (p < last && ip == p->start)
        return p;
    
    if (p > ranges->ranges)
        p--;
    
    return (p->start <= ip && ip <= p->end) ? p : NULL;
    
}

